from rest_framework import serializers
from s3_file_uploads.constants import ACCESS_CONTROL_TYPES, PRIVATE
from s3_file_uploads.models import UploadedFile


class UploadedFileSerializer(serializers.ModelSerializer):

    file_name = serializers.CharField(source='file.name', read_only=True)
    file = serializers.URLField(source='get_download_url', read_only=True)

    class Meta:
        model = UploadedFile
        fields = [
            'id',
            'created',
            'modified',
            'file_key',
            'file',
            'filename',
            'file_name',
            'file_path',
            'user',
        ]
        read_only_fields = [
            'id',
            'modfied',
            'created',
            'file_name',
            'file_path',
            'file_key'
        ]

class AccessControlListSerializer(serializers.Serializer):
    acl = serializers.ChoiceField(choices=ACCESS_CONTROL_TYPES, default=PRIVATE)



from django.conf.urls import url
from s3_file_uploads import views


app_name = 's3_file_uploads'


urlpatterns = [
    url(r'^$', view=views.UploadedFileCreateView.as_view(), name='upload-file-create'),
    url(r'^(?P<file_id>[0-9a-f-]+)/$', view=views.UploadedFileFetchView.as_view(), name='upload-file-fetch'),
    url(r'^(?P<file_id>[0-9a-f-]+)/complete/$', view=views.UploadedFileUploadCompleteView.as_view(), name='upload-file-complete'),
]



# Generated by Django 2.2 on 2019-08-14 06:10

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('s3_file_uploads', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='uploadedfile',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='uploaded_files', to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from rest_framework import generics, status, views
from rest_framework.response import Response
from django.urls import reverse
from s3_file_uploads.models import UploadedFile
from s3_file_uploads.serializers import UploadedFileSerializer, AccessControlListSerializer


class UploadedFileCreateView(generics.CreateAPIView):
    serializer_class = UploadedFileSerializer

    def get_queryset(self):
        return UploadedFile.objects.filter(user=self.request.user)

    def create(self, request, *args, **kwargs):
        acl_serialiser = AccessControlListSerializer(data=request.data)
        acl_serialiser.is_valid(raise_exception=True)
        file_serializer = self.get_serializer(
            data={**request.data, 'user': request.user.id}
        )
        file_serializer.is_valid(raise_exception=True)
        self.perform_create(file_serializer)
        headers = self.get_success_headers(file_serializer.data)
        uploaded_file = file_serializer.instance
        uploaded_file_data = UploadedFileSerializer(instance=uploaded_file).data
        uploaded_file_data['upload_form'] = uploaded_file.get_upload_form(
            acl_type=acl_serialiser.validated_data['acl']
        )
        uploaded_file_data['complete_url'] = request.build_absolute_uri(
            reverse('s3_file_uploads:upload-file-complete', args=(str(uploaded_file.id),))
        )
        return Response(uploaded_file_data, status=status.HTTP_201_CREATED, headers=headers)


class UploadedFileUploadCompleteView(views.APIView):
    def post(self, request, file_id):
        uploaded_file = get_object_or_404(
            UploadedFile,
            id=file_id,
            file_upload_state=UploadedFile.AWAIT_COMPLETE
        )
        uploaded_file.completed_upload()
        return Response({
            'status': UploadedFile.COMPLETED
        })


class UploadedFileFetchView(views.APIView):
    def get(self, request, file_id):
        uploaded_file = get_object_or_404(
            UploadedFile,
            id=file_id
        )
        return HttpResponseRedirect(uploaded_file.get_download_url())



# ACL Types

PRIVATE = 'private'
PUBLIC_READ = 'public-read'
PUBLIC_READ_WRITE = 'public-read-write'
AUTHENTICATED_READ = 'authenticated-read'

ACCESS_CONTROL_TYPES = [PRIVATE, PUBLIC_READ, PUBLIC_READ_WRITE, AUTHENTICATED_READ]



# Generated by Django 2.2 on 2019-04-08 10:55

from django.db import migrations, models
import django_fsm
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UploadedFile',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('file_key', models.CharField(blank=True, max_length=255)),
                ('filename', models.CharField(blank=True, max_length=255)),
                ('file_upload_state', django_fsm.FSMField(default='NEW', max_length=50)),
            ],
            options={
                'ordering': ['-created'],
            },
            bases=(django_fsm.ConcurrentTransitionMixin, models.Model),
        ),
    ]



import uuid
from django.contrib.auth import get_user_model
from django.db import models
from django.urls import reverse
from django_fsm import FSMField, transition, ConcurrentTransitionMixin

from s3_file_uploads.aws import S3AssetHandler


USER_MODEL = get_user_model()


class UploadedFile(ConcurrentTransitionMixin, models.Model):
    NEW = 'NEW'
    AWAIT_COMPLETE = 'AWAIT_COMPLETE'
    COMPLETED = 'COMPLETED'
    UPLOAD_STATES = [
        (NEW, 'New'),
        (AWAIT_COMPLETE, 'Awaiting Completion'),
        (COMPLETED, 'Completed'),
    ]

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    created = models.DateTimeField(auto_now_add=True, db_index=True)
    modified = models.DateTimeField(auto_now=True)

    file_key = models.CharField(max_length=255, blank=True)
    filename = models.CharField(max_length=255, blank=True)
    file_upload_state = FSMField(default=NEW)

    user = models.ForeignKey(
        USER_MODEL,
        related_name='uploaded_files',
        on_delete=models.SET_NULL,
        null=True
    )

    class Meta:
        ordering = ['-created']

    @property
    def file_path(self):
        return reverse('s3_file_uploads:upload-file-fetch', args=(str(self.id),))

    def get_file_key(self):
        return str(self.file_key or self.id)

    @property
    def asset_handler(self):
        return S3AssetHandler(self.get_file_key())

    def get_upload_form(self, *args, **kwargs) -> dict:
        form = self.trans_get_upload_form(*args, **kwargs)
        self.save()
        return form

    @transition(field=file_upload_state, source=[NEW, AWAIT_COMPLETE], target=AWAIT_COMPLETE)
    def trans_get_upload_form(self, *args, **kwargs) -> dict:
        return self.asset_handler.get_upload_form(*args, **kwargs)

    def completed_upload(self) -> None:
        self.trans_completed_upload()
        self.save()

    @transition(field=file_upload_state, source=AWAIT_COMPLETE, target=COMPLETED)
    def trans_completed_upload(self) -> None:
        pass

    @transition(field=file_upload_state, source=[NEW, COMPLETED], target=AWAIT_COMPLETE)
    def permit_reupload(self) -> None:
        # by default we don't allow re-uploading, but you can explicitly call
        # this to allow it. Useful for when something invalid was uploaded and
        # you want them to be able to upload it again.
        pass

    def get_view_url(self) -> str:
        return self.asset_handler.get_view_url(str(self.filename))

    def get_download_url(self) -> str:
        return self.asset_handler.get_download_url(str(self.filename))



"""
Create short term uploads locations on S3 (client will do uploading).
Also get short term view urls.
"""
from django.conf import settings
import boto3
from io import BytesIO
from s3_file_uploads.constants import PRIVATE


class S3AssetHandler:
    UPLOAD_ENDPOINT_EXPIRES = 300
    DOWNLOAD_ENDPOINT_EXPIRES = 300

    def __init__(self, id: str) -> None:
        self.id = str(id)  # coerce just in case someone passes in a UUID()
        self.s3 = boto3.client('s3')

    def get_upload_form(self, acl_type=PRIVATE, expiry=UPLOAD_ENDPOINT_EXPIRES) -> dict:
        # NOTE: Used because it's the only way to enforce upload
        #       size on S3.
        return self.s3.generate_presigned_post(
            settings.AWS_BUCKET_NAME,
            self.id,
            Fields={'acl': acl_type},
            Conditions=[
                {'acl': acl_type},
                ["content-length-range", 1, settings.MAX_FILE_UPLOAD_SIZE],
            ],
            ExpiresIn=expiry
        )

    def get_view_url(self, filename: str, duration=DOWNLOAD_ENDPOINT_EXPIRES) -> str:
        return self.s3.generate_presigned_url(
            'get_object',
            Params={
                'Bucket': settings.AWS_BUCKET_NAME,
                'Key': self.id,
                'ResponseContentDisposition': "filename=\"{}\"".format(
                    filename
                )
            },
            ExpiresIn=duration
        )

    def get_download_url(self, filename: str, duration=DOWNLOAD_ENDPOINT_EXPIRES) -> str:
        return self.s3.generate_presigned_url(
            'get_object',
            Params={
                'Bucket': settings.AWS_BUCKET_NAME,
                'Key': self.id,
                'ResponseContentDisposition': "attachment; filename=\"{}\"".format(
                    filename
                )
            },
            ExpiresIn=duration
        )

    def open_content(self) -> BytesIO:
        # NOTE: Downloads file into memory, not suitable for large files!
        data = BytesIO()
        self.s3.download_fileobj(
            Bucket=settings.AWS_BUCKET_NAME,
            Key=self.id,
            Fileobj=data
        )
        return data

    def upload_data(self, content: bytes):
        self.s3.put_object(
            ACL='private',
            Bucket=settings.AWS_BUCKET_NAME,
            Key=self.id,
            Body=content,
            ContentType='application/pdf',
            ContentDisposition='attachment'
        )



from rest_framework import serializers

from s3_file_uploads.models import UploadedFile


class UploadedFilePrimaryKeyRelatedField(serializers.PrimaryKeyRelatedField):

    def get_queryset(self):
        request = self.context['request']
        return UploadedFile.objects.filter(user=request.user)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
from setuptools import setup
import re
import os
import sys


name = 'django-s3-file-upload'
package = 's3_file_uploads'
description = 'Upload files from the browser to S3 - server side implementation'
url = 'https://www.dabapps.com'
author = 'DabApps'
author_email = 'engineering@dabapps.com'
license = 'BSD'
install_requires = [
    'boto3==1.18.63',
    'Django>=3.2',
    'django-fsm==2.8.0'
]
long_description_content_type="text/markdown",
long_description = """
    Server side implementation for uploading files straight from the browser to S3.

    Package provides endpoints for
        - getting an S3 signed url to upload files to
        - marking an upload as complete on the server
        - getting an S3 endpoint to download files from
    """



def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': get_version(package)}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s'" % args)
    print("  git push --tags")
    sys.exit()


setup(
    name=name,
    version=get_version(package),
    url=url,
    license=license,
    description=description,
    long_description=long_description,
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=install_requires,
    classifiers=[
    ]
)
